package eu.mhsl.craftattack.spawn.appliances.outlawed;

import eu.mhsl.craftattack.spawn.Main;
import eu.mhsl.craftattack.spawn.aggregates.displayName.DisplayName;
import eu.mhsl.craftattack.spawn.appliance.Appliance;
import eu.mhsl.craftattack.spawn.appliance.ApplianceCommand;
import eu.mhsl.craftattack.spawn.appliances.whitelist.Whitelist;
import eu.mhsl.craftattack.spawn.config.Configuration;
import eu.mhsl.craftattack.spawn.util.text.DisconnectInfo;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.event.HoverEvent;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import org.bukkit.event.Listener;
import org.jetbrains.annotations.NotNull;

import java.util.List;
import java.util.Map;
import java.util.WeakHashMap;

public class Outlawed extends Appliance {
    public enum Status {
        DISABLED,
        VOLUNTARILY,
        FORCED
    }

    private final Map<Player, Status> playerStatusMap = new WeakHashMap<>();
    private final String voluntarilyEntry = "voluntarily";
    public Outlawed() {
        super("outlawed");
        Bukkit.getScheduler().runTaskTimerAsynchronously(
                Main.instance(),
                () -> {
                    playerStatusMap.forEach((player, status) -> {
                        if(status != Status.FORCED) return;
                        try {
                            Main.instance().getAppliance(Whitelist.class).integrityCheck(player);
                        } catch (DisconnectInfo.Throwable e) {
                            Bukkit.getScheduler().runTask(Main.instance(), () -> e.getDisconnectScreen().applyKick(player));
                        }
                    });
                },
                20*60,
                20*60*5
        );
    }

    public void switchLawStatus(Player player) throws OutlawForcedException {
        if(getLawStatus(player).equals(Status.FORCED)) {
            throw new OutlawForcedException("Dein Vogelfreistatus wurde als Strafe auferlegt und kann daher nicht verändert werden.");
        }

        setLawStatus(player, isOutlawed(player) ? Status.DISABLED : Status.VOLUNTARILY);
    }

    public void updateForcedStatus(Player player, boolean forced) {
        setLawStatus(player, forced ? Status.FORCED : getLawStatus(player) == Status.FORCED ? Status.DISABLED : getLawStatus(player));
    }

    public Status getLawStatus(Player player) {
        Status status = playerStatusMap.computeIfAbsent(player, p -> {
            if(localConfig().getStringList(voluntarilyEntry).contains(p.getUniqueId().toString())) return Status.VOLUNTARILY;
            return Status.DISABLED;
        });
        return status;
    }

    public void setLawStatus(Player player, Status status) {
        playerStatusMap.put(player, status);
        Main.instance().getAppliance(DisplayName.class).update(player);

        List<String> newList = localConfig().getStringList(voluntarilyEntry);
        if(status.equals(Status.VOLUNTARILY)) {
            newList.add(player.getUniqueId().toString());
        } else {
            newList.remove(player.getUniqueId().toString());
        }
        localConfig().set(voluntarilyEntry, newList.stream().distinct().toList());
        Configuration.saveChanges();

    }

    public boolean isOutlawed(Player player) {
        return getLawStatus(player) != Status.DISABLED;
    }

    public Component getStatusDescription(Status status) {
        return switch (status) {
            case DISABLED -> Component.text("Vogelfreistatus inaktiv: ", NamedTextColor.GREEN)
                    .append(Component.text("Es gelten die Standart Regeln!", NamedTextColor.GOLD));

            case VOLUNTARILY, FORCED -> Component.text("Vogelfreistatus aktiv: ", NamedTextColor.RED)
                    .append(Component.text("Du darfst von jedem angegriffen und getötet werden!", NamedTextColor.GOLD));
        };
    }

    public Component getNamePrefix(Player player) {
        if(isOutlawed(player)) {
            return Component.text("[☠]", NamedTextColor.RED)
                    .hoverEvent(HoverEvent.showText(Component.text("Vogelfreie Spieler dürfen ohne Grund angegriffen werden!")));
        }

        return null;
    }

    @Override
    protected @NotNull List<ApplianceCommand<?>> commands() {
        return List.of(new OutlawedCommand());
    }

    @Override
    protected @NotNull List<Listener> eventHandlers() {
        return List.of(new OutlawedReminderListener());
    }
}
